@@ -6,8 +6,8 @@ describe ChildProcess do
   it "returns self when started" do
     process = sleeping_ruby
 
-    process.start.should == process
-    process.should be_started
+    expect(process.start).to eq process
+    expect(process).to be_alive
   end
 
   # We can't detect failure to execve() when using posix_spawn() on Linux
@@ -30,28 +30,35 @@ describe ChildProcess do
     process = exit_with(1).start
     process.wait
 
-    process.should be_crashed
+    expect(process).to be_crashed
   end
 
   it "knows if the process didn't crash" do
     process = exit_with(0).start
     process.wait
 
-    process.should_not be_crashed
+    expect(process).to_not be_crashed
   end
 
   it "can wait for a process to finish" do
     process = exit_with(0).start
     return_value = process.wait
 
-    process.should_not be_alive
-    return_value.should == 0
+    expect(process).to_not be_alive
+    expect(return_value).to eq 0
+  end
+
+  it 'ignores #wait if process already finished' do
+    process = exit_with(0).start
+    sleep 0.01 until process.exited?
+
+    expect(process.wait).to eql 0
   end
 
   it "escalates if TERM is ignored" do
     process = ignored('TERM').start
     process.stop
-    process.should be_exited
+    expect(process).to be_exited
   end
 
   it "accepts a timeout argument to #stop" do
@@ -66,9 +73,8 @@ describe ChildProcess do
         process.wait
       end
 
-      file.rewind
-      child_env = eval(file.read)
-      child_env['INHERITED'].should == 'yes'
+      child_env = eval rewind_and_read(file)
+      expect(child_env['INHERITED']).to eql 'yes'
     end
   end
 
@@ -78,13 +84,12 @@ describe ChildProcess do
       process.environment['CHILD_ONLY'] = '1'
       process.start
 
-      ENV['CHILD_ONLY'].should be_nil
+      expect(ENV['CHILD_ONLY']).to be_nil
 
       process.wait
-      file.rewind
 
-      child_env = eval(file.read)
-      child_env['CHILD_ONLY'].should == '1'
+      child_env = eval rewind_and_read(file)
+      expect(child_env['CHILD_ONLY']).to eql '1'
     end
   end
 
@@ -97,11 +102,10 @@ describe ChildProcess do
         process.start
         process.wait
 
-        file.rewind
-        child_env = eval(file.read)
+        child_env = eval rewind_and_read(file)
 
-        child_env['INHERITED'].should == 'yes'
-        child_env['CHILD_ONLY'].should == 'yes'
+        expect(child_env['INHERITED']).to eq 'yes'
+        expect(child_env['CHILD_ONLY']).to eq 'yes'
       end
     end
   end
@@ -114,10 +118,22 @@ describe ChildProcess do
       process.start
 
       process.wait
-      file.rewind
 
-      child_env = eval(file.read)
-      child_env.should_not have_key('CHILDPROCESS_UNSET')
+      child_env = eval rewind_and_read(file)
+      expect(child_env).to_not have_key('CHILDPROCESS_UNSET')
+    end
+  end
+
+  it 'does not see env vars unset in parent' do
+    Tempfile.open('env-spec') do |file|
+      ENV['CHILDPROCESS_UNSET'] = nil
+      process = write_env(file.path)
+      process.start
+
+      process.wait
+
+      child_env = eval rewind_and_read(file)
+      expect(child_env).to_not have_key('CHILDPROCESS_UNSET')
     end
   end
 
@@ -129,13 +145,13 @@ describe ChildProcess do
       process = write_argv(file.path, *args).start
       process.wait
 
-      file.rewind
-      file.read.should == args.inspect
+      expect(rewind_and_read(file)).to eql args.inspect
     end
   end
 
   it "lets a detached child live on" do
     pending "how do we spec this?"
+    fail
   end
 
   it "preserves Dir.pwd in the child" do
@@ -151,8 +167,7 @@ describe ChildProcess do
 
       process.wait
 
-      file.rewind
-      file.read.should == expected_dir
+      expect(rewind_and_read(file)).to eq expected_dir
     end
   end
 
@@ -163,8 +178,7 @@ describe ChildProcess do
       process = write_argv(file.path, *args).start
       process.wait
 
-      file.rewind
-      file.read.should == args.inspect
+      expect(rewind_and_read(file)).to eq args.inspect
     end
   end
 
@@ -172,14 +186,14 @@ describe ChildProcess do
     path = File.expand_path('foo bar')
 
     with_executable_at(path) do |proc|
-      proc.start.should == proc
-      proc.should be_started
+      expect(proc.start).to eq proc
+      expect(proc).to be_alive
     end
   end
 
   it "times out when polling for exit" do
     process = sleeping_ruby.start
-    lambda { process.poll_for_exit(0.1) }.should raise_error(ChildProcess::TimeoutError)
+    expect { process.poll_for_exit(0.1) }.to raise_error(ChildProcess::TimeoutError)
   end
 
   it "can change working directory" do
@@ -196,11 +210,35 @@ describe ChildProcess do
         process.start
         process.wait
 
-        file.rewind
-        file.read.should == dir
+        expect(rewind_and_read(file)).to eq dir
       end
 
-      Dir.pwd.should == orig_pwd
+      expect(Dir.pwd).to eq orig_pwd
     }
   end
+
+  it 'kills the full process tree', :process_builder => false do
+    Tempfile.open('kill-process-tree') do |file|
+      process = write_pid_in_sleepy_grand_child(file.path)
+      process.leader = true
+      process.start
+
+      pid = wait_until(30) do
+        Integer(rewind_and_read(file)) rescue nil
+      end
+
+      process.stop
+      wait_until(3) { expect(alive?(pid)).to eql(false) }
+    end
+  end
+
+  it 'releases the GIL while waiting for the process' do
+    time = Time.now
+    threads = []
+
+    threads << Thread.new { sleeping_ruby(1).start.wait }
+    threads << Thread.new(time) { expect(Time.now - time).to be < 0.5 }
+
+    threads.each { |t| t.join }
+  end
 end
