@@ -17,14 +17,11 @@ describe ChildProcess do
       process.io.stderr = err
 
       process.start
-      process.io.stdin.should be_nil
+      expect(process.io.stdin).to be_nil
       process.wait
 
-      out.rewind
-      err.rewind
-
-      out.read.should == "0\n"
-      err.read.should == "1\n"
+      expect(rewind_and_read(out)).to eq "0\n"
+      expect(rewind_and_read(err)).to eq "1\n"
     ensure
       out.close
       err.close
@@ -47,30 +44,26 @@ describe ChildProcess do
       process.start
       process.wait
 
-      out.rewind
-      out.read.should == "0\n"
+      expect(rewind_and_read(out)).to eq "0\n"
     ensure
       out.close
     end
   end
 
   it "pumps all output" do
-    10.times do |i|
-      process = echo
+    process = echo
 
-      out = Tempfile.new("duplex")
+    out = Tempfile.new("pump")
 
-      begin
-        process.io.stdout = out
+    begin
+      process.io.stdout = out
 
-        process.start
-        process.poll_for_exit(exit_timeout)
+      process.start
+      process.wait
 
-        out.rewind
-        out.read.should == "hello\n"
-      ensure
-        out.close
-      end
+      expect(rewind_and_read(out)).to eq "hello\n"
+    ensure
+      out.close
     end
   end
 
@@ -91,8 +84,7 @@ describe ChildProcess do
 
       process.poll_for_exit(exit_timeout)
 
-      out.rewind
-      out.read.should == "hello world\n"
+      expect(rewind_and_read(out)).to eq "hello world\n"
     ensure
       out.close
     end
@@ -113,27 +105,26 @@ describe ChildProcess do
       process.start
 
       stdin = process.io.stdin
-      lf = ChildProcess.windows? ? "\r\n" : "\n"
 
       stdin.puts "hello"
       stdin.flush
-      wait_until { rewind_and_read(out_receiver).should == "hello#{lf}" }
+      wait_until { expect(rewind_and_read(out_receiver)).to match(/\Ahello\r?\n\z/m) }
 
       stdin.putc "n"
       stdin.flush
-      wait_until { rewind_and_read(out_receiver).should == "hello#{lf}n" }
+      wait_until { expect(rewind_and_read(out_receiver)).to match(/\Ahello\r?\nn\z/m) }
 
       stdin.print "e"
       stdin.flush
-      wait_until { rewind_and_read(out_receiver).should == "hello#{lf}ne" }
+      wait_until { expect(rewind_and_read(out_receiver)).to match(/\Ahello\r?\nne\z/m) }
 
       stdin.printf "w"
       stdin.flush
-      wait_until { rewind_and_read(out_receiver).should == "hello#{lf}new" }
+      wait_until { expect(rewind_and_read(out_receiver)).to match(/\Ahello\r?\nnew\z/m) }
 
       stdin.write "\nworld\n"
       stdin.flush
-      wait_until { rewind_and_read(out_receiver).should == "hello#{lf}new#{lf}world#{lf}" }
+      wait_until { expect(rewind_and_read(out_receiver)).to match(/\Ahello\r?\nnew\r?\nworld\r?\n\z/m) }
 
       stdin.close
       process.poll_for_exit(exit_timeout)
@@ -177,7 +168,7 @@ describe ChildProcess do
     out = stdout.read
     err = stderr.read
 
-    [out, err].should == %w[stdout stderr]
+    expect([out, err]).to eq %w[stdout stderr]
   end
 
   it "can set close-on-exec when IO is inherited" do
@@ -208,10 +199,20 @@ describe ChildProcess do
       process.start
       process.wait
 
-      out.rewind
-      out.read.size.should == 3000
+      expect(rewind_and_read(out).size).to eq 3000
     ensure
       out.close
     end
   end
+
+  it 'should not inherit stdout and stderr by default' do
+    cap = capture_std do
+      process = echo
+      process.start
+      process.wait
+    end
+
+    expect(cap.stdout).to eq ''
+    expect(cap.stderr).to eq ''
+  end
 end
